/**
 * Created by neither on 15/3/30.
 */
(function(QT){

    var is_debug = /debug=true/.test(location.href);

    var _log = function () {
        if (!is_debug && !window.DEBUG) return;
        console.log.apply(console, Array.prototype.slice.apply(arguments));
    };

    var __cachedTask = {},
        doc = document;

    var taskHelper = {

        clearTask: function(sid){
            delete __cachedTask[sid];
        },

        bindCallback: function(name, callback, context){

            context['on' + name.substring(0, 1).toUpperCase() + name.substring(1)] = function(rs){
                clearTimeout(context.timeoutId);
                if(Object.prototype.toString.call(callback) == '[object Function]'){
                    callback.call(context, rs, context.cmd, context.param);
                    taskHelper.clearTask(context.id);
                }
            };

            return context;
        }
    };


    function Task(SDKGroup, config){
        this.id = workbench.createSequenceId();
        this.SDKGroup = SDKGroup;
        this.cmd = config.cmd || '';
        this.config = config || {};
        this.param = config.param || {};
    }

    Task.prototype = {

        bindCallback: taskHelper.bindCallback,

        success: function(callback){

            if(!callback) return this;

            this.bindCallback('success', callback, this);

            return this;
        },

        failure: function(callback){

            if(!callback) return this;

            this.bindCallback('failure', callback, this);

            return this;
        },

        timeout: function(time){

            if(!time || isNaN(time)) return this;

            var self = this;

            this.timeoutId = setTimeout(function(){
                self.onFailure && self.onFailure({
                    ret: 'TIMEOUT'
                });
            }, parseInt(time));

            return this;
        },

        run: function(){

            var param = this.param;

            if(Object.prototype.toString.call(param) !== '[object String]'){
                try{
                    param = JSON.stringify(param);
                }catch(e){}
            }

            this.SDKGroup.invoke(this.id, this.cmd, param);

        },

        request: function(requestMethod){
            requestMethod = requestMethod || 'CommitHttpRequest';
            var config = {
                url: this.config.url,
                method: this.config.method || 'get',
                param: this.param
            };

            if(Object.prototype.toString.call(config) !== '[object String]'){
                try{
                    config = JSON.stringify(config);
                }catch(e){}
            }

            workbench.httprequest.invoke(this.id, requestMethod, config);
        }
    };

    var TaskFactory = {

        getTask: function(SDKGroup, config){

            var task = new Task(SDKGroup, config);

            __cachedTask[task.id] = task;

            return task;

        }
    };

    // public
    QT.invokeHelper = function(SDKGroup, config){

        TaskFactory.getTask(SDKGroup, config)
            .success(config.success)
            .failure(config.failure)
            .timeout(config.timeout)
            .run();

    };

    /**
     * 通过客户端代理http请求
     * @param config
     * {
     *  url: '',
     *  method: '',
     *  param: {key: value}
     * }
     * @param requestMethod 请求的类型，默认为'CommitHttpRequest'
     */
    QT.ajax = function(config, requestMethod){
        TaskFactory.getTask(null, config)
            .success(config.success)
            .failure(config.failure)
            .timeout(config.timeout)
            .request(requestMethod);
    };

    function loop(newNamespace, originalNamespace, apiList){
        if(Object.prototype.toString.call(apiList) == '[object Object]'){
            for(var k in apiList){
                if(!originalNamespace[k]){
                    console.error(originalNamespace + '下没有' + k + '属性!');
                }else{
                    !newNamespace[k] && (newNamespace[k] = {});
                    loop(newNamespace[k], originalNamespace[k], apiList[k]);
                }

            }
        }else if(Object.prototype.toString.call(apiList) == '[object Array]'){
            apiList.forEach(function(item){

                if(typeof item === 'string'){
                    return (function(wb, cmd){
                        newNamespace[cmd] = function(param, success, failure, timeout){
                            if(arguments.length == 0){
                                param = success = failure = timeout = null;
                            }else{

                                // 不传param的情况下
                                if(Object.prototype.toString.call(param) == '[object Function]'){
                                    success = param;
                                    failure = success;
                                    timeout = failure;
                                    param = null;
                                }
                                if(Object.prototype.toString.call(timeout) == '[object Function]'){
                                    timeout = null;
                                }
                            }

                            QT.invokeHelper(wb, {
                                cmd: cmd,
                                param: param,
                                success: success,
                                failure: failure,
                                timeout: timeout
                            });
                        }
                    })(originalNamespace, item);
                }else{
                    loop(newNamespace, originalNamespace, item);
                }

            });
        }
    }

    QT.apiAdaptor = function(apiList){
        loop(QT, window.workbench || {}, apiList);
    };

    var __eventCache = {};

    QT.on = function(eventName, callback){
        var events = eventName.split(/\s+/);

        events.forEach(function(event){
            if(!__eventCache[event]){

                __eventCache[event] = {};

                __eventCache[event].callbacks = [];
                __eventCache[event].handle = function(ev){
                    __eventCache[event].callbacks.forEach(function(cb){
                        cb.call(this, ev.data);
                    });
                };

                document.addEventListener(event, __eventCache[event].handle, false);

            }

            if(__eventCache[event].callbacks.indexOf(callback) == -1){
                __eventCache[event].callbacks.push(callback);
            }

        });

        return this;
    };

    QT.off = function(eventName, callback){

        var events = eventName.split(/\s+/);

        events.forEach(function(event){
            if(__eventCache[event]){
                if(!callback){
                    document.removeEventListener(event, __eventCache[event].handle, false);
                    delete __eventCache[event];
                }else{
                    __eventCache[event].callbacks.forEach(function(cb, i){
                        if(cb === callback){
                            __eventCache[event].callbacks.splice(i, 1);
                        }
                    });

                    if(!__eventCache[event].length){
                        delete __eventCache[event];
                    }

                }
            }
        });

        return this;
    };

    function onInvokeNotify(sid, status, rs){

        var task = __cachedTask[sid];

        if(!task) return;

        try{
            rs = JSON.parse(rs);
        }catch(e){}

        // success
        if(status === 0){

            task.onSuccess && task.onSuccess(rs);

        }else{

            task.onFailure && task.onFailure(rs);
        }
    }

    function onEventNotify(sid, eventName, status, rs){

        if(status === 0){

            _log('%cEvent "' + eventName + '" fired', 'color:blue');

            // 当native需要通知js的时候（通信），用触发事件的方式进行
            var ev = doc.createEvent('HTMLEvents');
            ev.initEvent(eventName, false, true);

            try{
                rs = JSON.parse(rs);
            }catch(e){}
            _log(rs);

            ev.data = typeof rs !== 'undefined' ? rs : {};
            doc.dispatchEvent(ev);
        }
    }

    function onPushNotify(sid, pushName, status, rs){
        // 当native需要通知js的时候（通信），用触发事件的方式进行
        var ev = doc.createEvent('HTMLEvents');
        ev.initEvent(pushName, false, true);

        _log('%cPush "' + pushName + '" fired', 'color:blue');

        try{
            rs = JSON.parse(rs);
        }catch(e){}

        _log(rs);

        ev.data = rs || {};
        doc.dispatchEvent(ev);
    }

    // 暴露全局函数给客户端调用
    window.onInvokeNotify = onInvokeNotify;
    window.onEventNotify = onEventNotify;
    window.onPushNotify = onPushNotify;

})(window.QT || (window.QT = {}));

